; Source name		:	BiosTest.asm
; Executable name	:	Biostest.com
; Code model		:	Real mode flat model
; Version			:	1.0
; Created date		:	7.4.2008
; Last update		:	7.4.2008
; Author			:	Tomi Tilli
; Assembler			:	Yasm 0.61
; Description		:	Program to test if BIOS functions will force
;						interrupts on.
;						BIOS functions are called with far call.
;						Known values will be pushed to stack and loaded
;						to flags before function call.
;						After bios function returns, returned flags will be
;						checked for interrupt flag.

CPU 	286						; Only allow instructions supported by 286
BITS 	16						; Set 16 bit code generation
ORG 	100h					; Set code start address to 100h (.com file)

; Section containing code
SECTION .text

;--------------------------------------------------------------------
; Program start
;--------------------------------------------------------------------	
START:
	; Print default flags
	mov		dx, g_strStackFlag	; Load stack flag string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	mov		ax, STACKFLAGS		; Load flags to AX
	xchg	ah, al				; Write high byte first
	call	WriteByteInAL
	mov		al, ah				; Copy flags low to AL
	call	WriteByteInAL
	mov		dx, g_strDefFlag	; Load default flag string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	mov		ax, DEFFLAGS		; Load flags to AX
	xchg	ah, al				; Write high byte first
	call	WriteByteInAL
	mov		al, ah				; Copy flags low to AL
	call	WriteByteInAL
	call	WriteNewline
	
	; Load zero to ES
	xor		ax, ax
	mov		es, ax
	
	; Int 11h test, GET EQUIPMENT LIST
	mov		dx, g_str11h		; Load header string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	call	WriteFlagsRet		; Write flags returned string
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*11h]
	call	WriteFlags			; Write flags
	
	; Int 12h test, GET MEMORY SIZE
	mov		dx, g_str12h		; Load header string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	call	WriteFlagsRet		; Write flags returned string
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*12h]
	call	WriteFlags			; Write flags
	
	; Int 13h tests (disk functions)
	mov		dx, g_str13h		; Load header string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	
	; Test Int 13h/AH=0h Reset Disk System
	mov		ah, 0
	call	WriteAH
	mov		dl, 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=1h GET STATUS OF LAST OPERATION
	mov		ah, 1
	call	WriteAH
	mov		dl, 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=2h READ SECTOR(S) INTO MEMORY
	mov		ah, 2
	call	WriteAH
	push	ds					; Push DS
	pop		es					; Pop DS to ES
	push	WORD 0
	pop		ds					; Pop 0 to DS
	mov		al, SECTORS			; Number of sectors to read
	mov		bx, g_rgbData		; Offset to buffer
	mov		cx, 1				; Cyl 0, sector 1
	mov		dx, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[4*13h]
	push	es					; Push old DS
	pop		ds					; Restore DS
	push	WORD 0
	pop		es					; Pop 0 back to ES
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=4h VERIFY DISK SECTOR(S)
	mov		ah, 4
	call	WriteAH
	mov		al, SECTORS			; Number of sectors to read
	mov		cx, 1				; Cyl 0, sector 1
	mov		dx, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=8h GET DRIVE PARAMETERS
	mov		ah, 8
	call	WriteAH
	mov		dl, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=9h INITIALIZE CONTROLLER WITH DRIVE PARAMETERS
	mov		ah, 9
	call	WriteAH
	mov		dl, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=Ch SEEK TO CYLINDER
	mov		ah, 0Ch
	call	WriteAH
	mov		cx, 1				; Cyl 0, sector 1
	mov		dx, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=Dh RESET HARD DISKS
	mov		ah, 0Dh
	call	WriteAH
	mov		dl, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=10h CHECK IF DRIVE READY
	mov		ah, 10h
	call	WriteAH
	mov		dl, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=11h RECALIBRATE DRIVE
	mov		ah, 11h
	call	WriteAH
	mov		dl, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=14h CONTROLLER INTERNAL DIAGNOSTIC
	mov		ah, 14h
	call	WriteAH
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	; Test Int 13h/AH=15h GET DISK TYPE
	mov		ah, 15h
	call	WriteAH
	mov		dl, 80h				; Head 0, drive 80h
	push	WORD DEFFLAGS
	popf
	push	WORD STACKFLAGS		; Simulate int
	call far	[es:4*13h]
	call	WriteFlags			; Write flags
	
	

	; Exit to DOS
	sti
	mov 	ax, 4C00h			; This dos function exits the program
	int 	21h					;  and returns control to DOS (error code 0)
	

;--------------------------------------------------------------------
; Writes flags register.
;
; WriteFlags
;	Parameters:
;		Nothing
;	Returns:
;		Nothing
;	Corrupts registers:
;		AX, DX
;--------------------------------------------------------------------
ALIGN 2
WriteFlags:
	pushf						; Push flags
	pushf						; Push flags second time
	pop		ax					; Pop flags to AX
	sti							; Enable interrupts
	xchg	ah, al				; Change to write high first
	call	WriteByteInAL		; Write high byte
	mov		al, ah				; Copy high byte to AL
	call	WriteByteInAL		; Write low byte
	
	; Write IF state
	pop		ax					; Pop flags to AX
	test	ax, BIT_IF			; IF set?
	jnz		.WrIf				;  If so, jump to write it
	call	WriteNewline		; Change line
	ret
	
ALIGN 2
.WrIf:
	mov		dx, g_strIFFlag		; Load interrupt string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	call	WriteNewline		; Change line
	ret
		

;--------------------------------------------------------------------
; Writes flags returned string without AH function.
;
; WriteFlagsRet
;	Parameters:
;		Nothing
;	Returns:
;		Nothing
;	Corrupts registers:
;		AX, DX
;--------------------------------------------------------------------
ALIGN 2
WriteFlagsRet:
	mov		dx, g_strRetFlagOnly	; Load Returned string
	mov		ah, 9h					; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h						; Write newline string
	ret


;--------------------------------------------------------------------
; Writes AH string.
;
; WriteAH
;	Parameters:
;		Nothing
;	Returns:
;		Nothing
;	Corrupts registers:
;		DX
;--------------------------------------------------------------------
ALIGN 2
WriteAH:
	push	ax
	mov		dx, g_strAH			; Load AH string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	pop		ax
	mov		al, ah				; Copy AH to AL
	call	WriteByteInAL
	push	ax
	mov		dx, g_strRetFlag	; Load Returned string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	pop		ax
	ret


;--------------------------------------------------------------------
; Changes line
;
; WriteNewline
;	Parameters:
;		Nothing
;	Returns:
;		Nothing
;	Corrupts registers:
;		AX, DX
;--------------------------------------------------------------------
ALIGN 2
WriteNewline:
	mov		dx, g_strNewline	; Load newline string
	mov		ah, 9h				; DOS WRITE STRING TO STANDARD OUTPUT
	int		21h					; Write newline string
	ret


;--------------------------------------------------------------------
; Writes hexadecimal in AL to current cursor location.
; Cursor location will be updated.
;
; WriteByteInAL
;	Parameters:
;		AL:		Byte to display
;	Returns:
;		Nothing
;	Corrupts registers:
;		Nothing
;--------------------------------------------------------------------
ALIGN 2
WriteByteInAL:
	; Save registers
	push	dx					; Store DX
	push	bx					; Store BX
	push	ax					; Store AX
	
	; Dublicate byte in AL to DH and DL
	mov		dh, al				; Copy byte to DH
	mov		dl, al				; Copy byte to DL
	and		dl, 0Fh				; Clear bits 7...4 from DL
	shr		dh, 4				; Shift bits 7...4 to DH
	
	; Write bits 7...4
	mov		bx, g_rgbHex		; Load HEX lookup offset to BX
	mov		al, dh				; Copy bits 7...4 to AL
	xlatb						; mov al, [bx+al]
	mov		ah, 0Eh				; Write in Teletype Mode
	xor		bh, bh				; Page index to BH
	int		10h					; Write character
	
	; Write bits 3...0
	mov		bx, g_rgbHex		; Load HEX lookup offset to BX
	mov		al, dl				; Copy bits 3...0 to AL
	xlatb						; mov al, [bx+al]
	xor		bh, bh				; Page index to BH
	int		10h					; Write character
	
	; Restore registers and return
	pop		ax					; Restore AX
	pop		bx					; Restore BX
	pop		dx					; Restore DX
	ret
	


; Section for initialized data
SECTION .data

; Default flags: Set DF and OF, clear IF
BIT_CF		EQU		(1<<0)
BIT_PF		EQU		(1<<2)
BIT_AF		EQU		(1<<4)
BIT_ZF		EQU		(1<<6)
BIT_SF		EQU		(1<<7)
BIT_TF		EQU		(1<<8)
BIT_IF		EQU		(1<<9)
BIT_DF		EQU		(1<<10)
BIT_OF		EQU		(1<<11)
MASK_ALW	EQU		(1<<1)		; Always set to 1
STACKFLAGS	EQU		(MASK_ALW | BIT_DF)
DEFFLAGS	EQU		(MASK_ALW | BIT_OF | BIT_SF | BIT_ZF | BIT_AF | BIT_PF)

; Sectors
SECTORS		EQU		2

; Strings
g_strStackFlag:		db "Stack flags: ", "$"
g_strDefFlag:		db ", default flags: ", "$"
g_str11h: 			db "Testing INT 11h, Get Equipment List", 13, 10, "$"
g_str12h: 			db "Testing INT 12h, Get Memory Size", 13, 10, "$"
g_str13h: 			db "Testing INT 13h, Disk Functions", 13, 10, "$"
g_strAH:			db "Testing AH=", "$"
g_strRetFlag:		db ", flags returned: ", "$"
g_strRetFlagOnly:	db "Testing, flags returned: ", "$"
g_strIFFlag:		db ", Interrupts are enabled!", "$"
g_strNewline:		db 13, 10, "$"

g_rgbHex:
	db		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
	db		'A', 'B', 'C', 'D', 'E', 'F'
	
; Buffer for Int 13h read function
g_rgbData:
	times 	(SECTORS * 512)	db 0h


